// UR5FFR Si5351 NanoVFO
#include <avr/power.h>
#include <avr/eeprom.h> 
#include "config.h"
#include "pins.h"
#include <i2c.h>
#include "TRX.h"
#include "disp_MAX7219.h"
#include <si5351a.h>

#ifdef VFO_SI5351
  Si5351 vfo5351;
#endif

#ifdef DISPLAY_MAX7219
  Display_MAX7219 disp;
#endif

Encoder encoder;
TRX trx;


uint16_t KeyLevels[] = {150,300,450,600,780,900};
InputAnalogKeypad keypad(PIN_ANALOG_KEYPAD, 6, KeyLevels);

struct _Settings SettingsDef[SETTINGS_COUNT] = {
  SETTINGS_DATA
};

uint16_t EEMEM eeSettingsVersion = 0;
int EEMEM eeSettings[SETTINGS_COUNT] = {0};  

int Settings[SETTINGS_COUNT] = {0};  

void writeSettings()
{
  eeprom_write_block(Settings, eeSettings, sizeof(Settings));
}

void resetSettings()
{
  for (uint8_t j=0; j < SETTINGS_COUNT; j++)
    Settings[j] = SettingsDef[j].def_value;
}

void readSettings()
{
  uint16_t ver;
  ver = eeprom_read_word(&eeSettingsVersion);
  if (ver == ((SETTINGS_VERSION << 8) | SETTINGS_COUNT))
    eeprom_read_block(Settings, eeSettings, sizeof(Settings));
  else {
    // fill with defaults
    resetSettings();
    writeSettings();
    ver = (SETTINGS_VERSION << 8) | SETTINGS_COUNT;
    eeprom_write_word(&eeSettingsVersion, ver);
  }
}
void setup()
{
  readSettings();
  keypad.setup();
    pinMode(PIN_IN_BAND0, INPUT_PULLUP);
    pinMode(PIN_IN_BAND1, INPUT_PULLUP);
    pinMode(PIN_IN_BAND2, INPUT_PULLUP);
    pinMode(PIN_IN_BAND3, INPUT_PULLUP);
    pinMode(PIN_IN_BAND4, INPUT_PULLUP);
    pinMode(PIN_IN_BAND5, INPUT_PULLUP);
    pinMode(PIN_IN_BAND6, INPUT_PULLUP);
    pinMode(PIN_IN_BAND7, INPUT_PULLUP);
    pinMode(PIN_IN_BAND8, INPUT_PULLUP);

  i2c_init();
#ifdef VFO_SI5351
  vfo5351.VCOFreq_Max = 800000000; // для использования "кривых" SI-шек с нестабильной генерацией
  // change for required output level
  vfo5351.setup(
    SI5351_CLK0_DRIVE,
    SI5351_CLK1_DRIVE,
    SI5351_CLK2_DRIVE
  );
  vfo5351.set_xtal_freq((SI5351_CALIBRATION/10000)*10000+Settings[ID_SI5351_XTAL]);
#endif  

  encoder.Setup();
  disp.setup();
  trx.StateLoad();
  analogReference(DEFAULT);
}

#include "freq_calc.h"

void UpdateBandCtrl()
{
  if (!digitalRead(PIN_IN_BAND0))  trx.SwitchToBand(0);
  if (!digitalRead(PIN_IN_BAND1))  trx.SwitchToBand(1);
  if (!digitalRead(PIN_IN_BAND2))  trx.SwitchToBand(2);
  if (!digitalRead(PIN_IN_BAND3))  trx.SwitchToBand(3);
  if (!digitalRead(PIN_IN_BAND4))  trx.SwitchToBand(4);
  if (!digitalRead(PIN_IN_BAND5))  trx.SwitchToBand(5);
  if (!digitalRead(PIN_IN_BAND6))  trx.SwitchToBand(6);
  if (!digitalRead(PIN_IN_BAND7))  trx.SwitchToBand(7);
  if (!digitalRead(PIN_IN_BAND8))  trx.SwitchToBand(8);


}

long last_event = 0;
long last_pool = 0;
uint8_t last_key = 0;
uint8_t first_call = 1;

void edit_item(uint8_t mi)
{
  int val = Settings[mi];
  if (SettingsDef[mi].id == 99) {
    #ifdef VFO_SI5351
      vfo5351.out_calibrate_freq();
    #endif

  }
  disp.DrawMenu(SettingsDef[mi].id,SettingsDef[mi].title,val,1);
  keypad.waitUnpress();
  while (1) {
    uint8_t key = keypad.Read();
    switch (key) {
      case 1:
        // save value
        Settings[mi] = val;
        writeSettings();
        #ifdef VFO_SI5351
          if (SettingsDef[mi].id == 99) vfo5351.set_xtal_freq((SI5351_CALIBRATION/10000)*10000+Settings[ID_SI5351_XTAL]);
        #endif
        return;
        break;
      case 2:
        // exit
        return;
    }
    long d=encoder.GetDelta();
    if (d < 0) {
      val -= SettingsDef[mi].step;
      if (val < SettingsDef[mi].min_value) val = SettingsDef[mi].min_value;
      disp.DrawMenu(SettingsDef[mi].id,SettingsDef[mi].title,val,1);
    } else if (d > 0) {
      val += SettingsDef[mi].step;
      if (val > SettingsDef[mi].max_value) val = SettingsDef[mi].max_value;
      disp.DrawMenu(SettingsDef[mi].id,SettingsDef[mi].title,val,1);
    }
  }
}

void show_menu()
{
  uint8_t mi=0;
  disp.DrawMenu(SettingsDef[mi].id,SettingsDef[mi].title,Settings[mi],0);
  keypad.waitUnpress();
  while (1) {
    uint8_t key = keypad.Read();
    switch (key) {
      case 1:
        if (SettingsDef[mi].id == 0) {
          resetSettings();
          writeSettings();
        } else // edit value
          edit_item(mi);
        disp.DrawMenu(SettingsDef[mi].id,SettingsDef[mi].title,Settings[mi],0);
        keypad.waitUnpress();
        break;
      case 2:
        // exit
        disp.clear();
        return;
        break;
    }
    long d=encoder.GetDelta();
    if (d < 0) {
      if (mi == 0) mi = SETTINGS_COUNT-1;
      else mi--;
      disp.DrawMenu(SettingsDef[mi].id,SettingsDef[mi].title,Settings[mi],0);
      delay(300);
      encoder.GetDelta();
    } else if (d > 0) {
      mi++;
      if (mi >= SETTINGS_COUNT) mi = 0;
      disp.DrawMenu(SettingsDef[mi].id,SettingsDef[mi].title,Settings[mi],0);
      delay(300);
      encoder.GetDelta();
    }
  }
}

void loop()
{
  if (first_call && (keypad.Read() == 1)) {
    // show menu on startup
    show_menu();
    keypad.waitUnpress();
  }
  first_call = 0;
    
  long delta = encoder.GetDelta();
  if (delta) {
    trx.ChangeFreq(delta);
  }

  if (millis()-last_pool > POOL_INTERVAL) {
    last_pool = millis();
    uint8_t key = keypad.Read();
    if (!last_key && key) {
      // somewhat pressed
      last_key = key;
      switch (key) {
        case 1:
          trx.NextBand();
          break;
        case 2:
        show_menu();
          keypad.waitUnpress();
          break;    
        case 3:
          break; 
        case 4:
          break; 
        case 5:
          break; 
        case 6:
          break;           
      }
    
    } else if (!key) {
      last_key = 0;
    }
   
    UpdateFreq();
    UpdateBandCtrl();
    disp.Draw(trx);
   
  }
  
  static long state_poll_tm = 0;
  if (millis()-state_poll_tm > 500) {
    static uint16_t state_hash = 0;
    static uint8_t state_changed = false;
    static long state_changed_tm = 0;
    uint16_t new_state_hash = trx.StateHash();
    if (new_state_hash != state_hash) {
      state_hash = new_state_hash;
      state_changed = true;
      state_changed_tm = millis();
    } else if (state_changed && (millis()-state_changed_tm > 5000)) {
      // save state
      trx.StateSave();
      state_changed = false;
    }
    state_poll_tm = millis();
  }
}
