#include <Arduino.h>
#ifndef CONFIG_H
#define CONFIG_H
#define LSB 0
#define USB 1

// Используем индикатор на основе MAX7219:
#define DISPLAY_MAX7219

// keypad pool interval in ms
#define POOL_INTERVAL   50

//---------Диапазоны частот

// Общее число диапазонов:
#define BAND_COUNT  9

extern const struct _Bands {
  uint8_t   mc;
  long  start, startSSB, end;
  uint8_t sideband;
} Bands[];

// Описания диапазонов (частотоы в Гц):
#define DEFINED_BANDS \
  {160,  1810000L,  1840000L,  2000000L, LSB}, \
  {80,   3500000L,  3600000L,  3800000L, LSB}, \
  {40,   7000000L,  7045000L,  7200000L, LSB}, \
  {30,  10100000L, 10130000L, 10150000L, LSB}, \
  {20,  14000000L, 14100000L, 14350000L, LSB}, \
  {17,  18068000L, 18110000L, 18168000L, LSB}, \
  {15,  21000000L, 21150000L, 21450000L, USB}, \
  {12,  24890000L, 24930000L, 25140000L, USB}, \
  {10,  28000000L, 28200000L, 29700000L, USB}

//-----------Настройки синтезатора

// Используем синтезатор частоты SI5351:
#define VFO_SI5351

// Частота тактового генератора SI5351  в Гц :
#define SI5351_CALIBRATION       27000000

// Первый гетеродин, режим "одной ПЧ".
// Первый гетеродин выше частоты приема при LSB и ниже при USB:
#define MODE_SINGLE_IF

// Частота ПЧ  (SSBDetectorFreq_USB), в Гц. 
#define SSBDetectorFreq_USB     8800000L

// Множители частоты для выходов SI5351 (х1,2,4):
const long CLK0_MULT = 1;
const long CLK1_MULT = 1;
const long CLK2_MULT = 1;

// Токи выходов Si5351. 0=2mA, 1=4mA, 2=6mA, 3=8mA
#define SI5351_CLK0_DRIVE   2
#define SI5351_CLK1_DRIVE   3
#define SI5351_CLK2_DRIVE   2

//-------- Настройки  энкодера

// Используем  оптический энкодер, 360 импульсов на 1 оборот:
#define ENCODER_OPTICAL
// Изменение частоты в Гц на один оборот в обычном режиме:
#define ENCODER_FREQ_LO_STEP      1000
// Изменение частоты в Гц на один оборот в ускоренном режиме:
#define ENCODER_FREQ_HI_STEP      15000
// Порог переключения в ускоренный режим, если частота изменится более чем на 
// ENCODER_FREQ_HI_LO_TRASH (Гц) за секунду, то переходим в ускоренный режим:
#define ENCODER_FREQ_HI_LO_TRASH  2000

#ifdef ENCODER_OPTICAL
#define ENCODER_ENABLE
// Количество импульсов на один оборот примененного оптического энкодера:
#define ENCODER_PULSE_PER_TURN    180

class Encoder {
  public:
  static void Setup();
  static long GetDelta();
  static void SetValue(long Value);
};

//---------- Блок  настроек устройства

struct _Settings {
  int id;
  int def_value;
  int min_value;
  int max_value;
  int step;
  char* title;
};

#define SETTINGS_COUNT  3
#define SETTINGS_DATA \
  {02, 8300, 5000, 11000, 1, "CLK1"},  /* Частота второго гетеродина (фиксиованная), от 5 до 11 МГц с шагом в 1 кГц. Дефолтное значение 8.3 МГц */\
  {03, 4, -20000, 20000, 1, "SI5351 XTAL"},  /* Подстройка частоты Si5351 под опорный генератор */\
  {0, 0, 0, 0, 0, "FULL RESET"} /* Очистка всех пользовательских  настроек синтезатора, обнуление EEPROM */\

// increase for reset stored to EEPROM settings values to default
#define SETTINGS_VERSION    0x02

// id for fast settings access
enum {
  ID_CLK1,
  ID_SI5351_XTAL
};

//-------------Настройка выводов и их состояние
#define PIN_ANALOG_KEYPAD   A6
#define PIN_IN_BAND0   17
#define PIN_IN_BAND1   16
#define PIN_IN_BAND2   15
#define PIN_IN_BAND3   14
#define PIN_IN_BAND4   13
#define PIN_IN_BAND5   12
#define PIN_IN_BAND6   11
#define PIN_IN_BAND7   10
#define PIN_IN_BAND8    9


#endif
#endif
