#include <i2c.h>
#include "pins.h"

uint8_t InputPullUpPin::Read() {
  if (pin == PIN_NC) return false;
  if (millis()-last_tm < 50) return last;
  uint8_t val = digitalRead(pin) == LOW;
  if (val != last) {
	  last = val;
	  last_tm = millis();
  }
  return val;
}

void InputPullUpPin::setup() {
  if (pin != PIN_NC) 
    pinMode(pin, INPUT_PULLUP); 
}

void InputAnalogKeypad::setup()
{
  if (pin != PIN_NC) 
    pinMode(pin, INPUT); 
}

uint8_t InputAnalogKeypad::Read()
{
  if (pin == PIN_NC) return 0;
  if ((millis()-last_tm) < 50) return last;
  uint16_t aval = 1024;
  for (byte i=5; i > 0; i--) {
    uint16_t v = analogRead(pin);
    if (v < aval) aval = v;
  }
  uint8_t val=0;
  while (aval > levels[val] && val < btn_cnt) val++;
  val++;
  if (val > btn_cnt) val = 0;
  if (val != last) {
    last = val;
    last_tm = millis();
  }
  return val;
}

void InputAnalogKeypad::waitUnpress()
{
  while (Read() != 0) delay(1);
  delay(50);
}

void InputAnalogPin::setup() {
  if (pin != PIN_NC) 
    pinMode(pin, INPUT); 
}

// read internal 1.1 source
int ReadV11Ref() 
{
  int acc = 0;

  analogReference(DEFAULT);
  // Read 1.1V reference against AVcc
  // set the reference to Vcc and the measurement to the internal 1.1V reference
  ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);

  delay(1); // otherwise unstable
  for (byte i=0; i < 3; i++) {
    ADCSRA |= _BV(ADSC); // Start conversion
    while (bit_is_set(ADCSRA,ADSC)); // measuring
    uint8_t low  = ADCL; // must read ADCL first - it then locks ADCH
    uint8_t high = ADCH; // unlocks both
    acc += (high<<8) | low;
  }
  return acc/3;
  delay(1);
}

int InputAnalogPin::Read() 
{
  int vref = ReadV11Ref(); 
  int new_value = 0;
  analogReference(DEFAULT);
  new_value += analogRead(pin);
  new_value += analogRead(pin);
  new_value += analogRead(pin);
  new_value = (long)new_value *1100L/vref/3;
  if (abs(new_value-value) > rfac)
    value=new_value;
  return value;
}

int InputAnalogPin::ReadRaw()
{
  int new_value = 0;
  analogReference(DEFAULT);
  new_value += analogRead(pin);
  new_value += analogRead(pin);
  new_value += analogRead(pin);
  new_value = new_value/3;
  if (abs(new_value-value) > rfac)
    value=new_value;
  return value;
}

void OutputBinPin::setup() 
{
  if (pin != PIN_NC) {
    pinMode(pin, OUTPUT);
    Write(def_value);
  }
}

void OutputBinPin::Write(uint8_t value) 
{
  if (pin != PIN_NC && state != value) {
	  digitalWrite(pin,value?(active_level == HIGH?HIGH:LOW):(active_level == HIGH?LOW:HIGH));
	  state = value;
  }
}
